"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EndianReader = exports.EndianType = void 0;
const smart_buffer_1 = require("smart-buffer");
var EndianType;
(function (EndianType) {
    EndianType[EndianType["LE"] = 0] = "LE";
    EndianType[EndianType["BE"] = 1] = "BE";
})(EndianType = exports.EndianType || (exports.EndianType = {}));
class EndianReader extends smart_buffer_1.SmartBuffer {
    constructor(options) {
        var _a;
        super(options);
        this.endianness = (_a = options === null || options === void 0 ? void 0 : options.endianness) !== null && _a !== void 0 ? _a : EndianType.BE;
    }
    static fromBuffer(buff) {
        return new EndianReader({ buff });
    }
    alignStream(alignment = 4) {
        const pos = this.readOffset;
        const mod = pos % alignment;
        if (mod !== 0) {
            this.readOffset += alignment - mod;
        }
    }
    readAlignedString() {
        const length = this.readInt32();
        if (length > 0 && length <= this.internalBuffer.length - this.readOffset) {
            const result = this.readString(length, 'utf8');
            this.alignStream(4);
            return result;
        }
        return '';
    }
    readArray(method, count) {
        if (typeof this[method] !== 'function') {
            throw new Error('Not a valid read method');
        }
        const result = [];
        if (count === undefined) {
            count = this.readInt32();
        }
        for (let n = 0; n < count; ++n) {
            result.push(this[method]());
        }
        return result;
    }
    readBoolean(offset) {
        return !!(this.readInt8(offset) & 1);
    }
    readSingle(offset) {
        switch (this.endianness) {
            case EndianType.LE:
                return super.readFloatLE(offset);
            case EndianType.BE:
                return super.readFloatBE(offset);
            default:
                throw new Error(`Invalid endianness: ${this.endianness}`);
        }
    }
    readInt16(offset) {
        switch (this.endianness) {
            case EndianType.LE:
                return super.readInt16LE(offset);
            case EndianType.BE:
                return super.readInt16BE(offset);
            default:
                throw new Error(`Invalid endianness: ${this.endianness}`);
        }
    }
    readUInt16(offset) {
        switch (this.endianness) {
            case EndianType.LE:
                return super.readUInt16LE(offset);
            case EndianType.BE:
                return super.readUInt16BE(offset);
            default:
                throw new Error(`Invalid endianness: ${this.endianness}`);
        }
    }
    readInt32(offset) {
        switch (this.endianness) {
            case EndianType.LE:
                return super.readInt32LE(offset);
            case EndianType.BE:
                return super.readInt32BE(offset);
            default:
                throw new Error(`Invalid endianness: ${this.endianness}`);
        }
    }
    readUInt32(offset) {
        switch (this.endianness) {
            case EndianType.LE:
                return super.readUInt32LE(offset);
            case EndianType.BE:
                return super.readUInt32BE(offset);
            default:
                throw new Error(`Invalid endianness: ${this.endianness}`);
        }
    }
    readInt64(offset) {
        switch (this.endianness) {
            case EndianType.LE:
                return super.readBigInt64LE(offset);
            case EndianType.BE:
                return super.readBigInt64BE(offset);
            default:
                throw new Error(`Invalid endianness: ${this.endianness}`);
        }
    }
    readUInt64(offset) {
        switch (this.endianness) {
            case EndianType.LE:
                return super.readBigUInt64LE(offset);
            case EndianType.BE:
                return super.readBigUInt64BE(offset);
            default:
                throw new Error(`Invalid endianness: ${this.endianness}`);
        }
    }
    readFloat(offset) {
        switch (this.endianness) {
            case EndianType.LE:
                return super.readFloatLE(offset);
            case EndianType.BE:
                return super.readFloatBE(offset);
            default:
                throw new Error(`Invalid endianness: ${this.endianness}`);
        }
    }
    readDouble(offset) {
        switch (this.endianness) {
            case EndianType.LE:
                return super.readDoubleLE(offset);
            case EndianType.BE:
                return super.readDoubleBE(offset);
            default:
                throw new Error(`Invalid endianness: ${this.endianness}`);
        }
    }
}
exports.EndianReader = EndianReader;
//# sourceMappingURL=EndianReader.js.map