"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubMesh = exports.VertexData = exports.BoneWeights4 = exports.ChannelInfo = exports.StreamInfo = exports.AABB = exports.GfxPrimitiveType = void 0;
const VersionTuple_1 = require("../VersionTuple");
const Vector_1 = require("./Vector");
var GfxPrimitiveType;
(function (GfxPrimitiveType) {
    GfxPrimitiveType[GfxPrimitiveType["kPrimitiveTriangles"] = 0] = "kPrimitiveTriangles";
    GfxPrimitiveType[GfxPrimitiveType["kPrimitiveTriangleStrip"] = 1] = "kPrimitiveTriangleStrip";
    GfxPrimitiveType[GfxPrimitiveType["kPrimitiveQuads"] = 2] = "kPrimitiveQuads";
    GfxPrimitiveType[GfxPrimitiveType["kPrimitiveLines"] = 3] = "kPrimitiveLines";
    GfxPrimitiveType[GfxPrimitiveType["kPrimitiveLineStrip"] = 4] = "kPrimitiveLineStrip";
    GfxPrimitiveType[GfxPrimitiveType["kPrimitivePoints"] = 5] = "kPrimitivePoints";
})(GfxPrimitiveType = exports.GfxPrimitiveType || (exports.GfxPrimitiveType = {}));
class AABB {
    constructor(reader) {
        this.center = Vector_1.readVector3(reader);
        this.extent = Vector_1.readVector3(reader);
    }
}
exports.AABB = AABB;
class StreamInfo {
    constructor(reader) {
        const version = reader.version;
        this.channelMask = reader.readUInt32();
        this.offset = reader.readUInt32();
        if (version < VersionTuple_1.makeVersion(4)) {
            //4.0 down
            this.stride = reader.readUInt32();
            this.align = reader.readUInt32();
        }
        else {
            this.stride = reader.readInt8();
            this.dividerOp = reader.readInt8();
            this.frequency = reader.readUInt16();
        }
    }
}
exports.StreamInfo = StreamInfo;
class ChannelInfo {
    constructor(reader) {
        this.stream = reader.readInt8();
        this.offset = reader.readInt8();
        this.format = reader.readInt8();
        this.dimension = reader.readInt8() & 0xf;
    }
}
exports.ChannelInfo = ChannelInfo;
class BoneWeights4 {
    constructor(reader) {
        this.weight = reader.readArray('readSingle', 4);
        this.boneIndex = reader.readArray('readInt32', 4);
    }
}
exports.BoneWeights4 = BoneWeights4;
class VertexData {
    constructor(reader) {
        var version = reader.version;
        if (version < VersionTuple_1.makeVersion(2018)) {
            //2018 down
            this.currentChannels = reader.readUInt32();
        }
        this.vertexCount = reader.readUInt32();
        if (version >= VersionTuple_1.makeVersion(4)) {
            //4.0 and up
            const channelsSize = reader.readInt32();
            this.channels = [];
            for (let n = 0; n < channelsSize; ++n) {
                this.channels[n] = new ChannelInfo(reader);
            }
        }
        if (version < VersionTuple_1.makeVersion(5)) {
            //5.0 down
            if (version < VersionTuple_1.makeVersion(4)) {
                this.streams = [];
            }
            else {
                this.streams = [];
            }
            for (let n = 0; n < this.streams.length; ++n) {
                this.streams[n] = new StreamInfo(reader);
            }
            if (version < VersionTuple_1.makeVersion(4)) {
                //4.0 down
                // TODO
                // this.getChannels(version);
            }
        } //5.0 and up
        else {
            // TODO
            // this.getStreams(version);
        }
        this.dataSize = reader.readArray('readUInt8');
        reader.alignStream();
    }
}
exports.VertexData = VertexData;
class SubMesh {
    constructor(reader) {
        const version = reader.version;
        this.firstByte = reader.readUInt32();
        this.indexCount = reader.readUInt32();
        this.topology = reader.readInt32();
        if (version < VersionTuple_1.makeVersion(4)) {
            this.triangleCount = reader.readUInt32();
        }
        if (version >= VersionTuple_1.makeVersion(2017, 3)) {
            this.baseVertex = reader.readUInt32();
        }
        if (version >= VersionTuple_1.makeVersion(3)) {
            this.firstVertex = reader.readUInt32();
            this.vertexCount = reader.readUInt32();
            this.localAABB = new AABB(reader);
        }
    }
}
exports.SubMesh = SubMesh;
//# sourceMappingURL=Mesh.js.map