/// <reference types="node" />
/// <reference types="ref-napi" />
import { EndianReader } from '../EndianReader';
import { ObjectReader } from '../ObjectReader';
import { PPtr } from './PPtr';
import { Matrix4x4 } from './Matrix';
import { BoneWeights4, SubMesh, VertexData } from './Mesh';
import { NamedObject } from './NamedObject';
import { Texture2D } from './Texture2D';
import { Vector2, Vector4, Vector3 } from './Vector';
import { BaseObject } from './BaseObject';
export declare enum SpritePackingRotation {
    kSPRNone = 0,
    kSPRFlipHorizontal = 1,
    kSPRFlipVertical = 2,
    kSPRRotate180 = 3,
    kSPRRotate90 = 4
}
export declare enum SpritePackingMode {
    kSPMTight = 0,
    kSPMRectangle = 1
}
export declare enum SpriteMeshType {
    kSpriteMeshTypeFullRect = 0,
    kSpriteMeshTypeTight = 1
}
export declare class Rectf {
    x: number;
    y: number;
    width: number;
    height: number;
    constructor(reader: EndianReader);
}
export declare class SecondarySpriteTexture {
    readonly texture: PPtr<Texture2D>;
    readonly name: string;
    constructor(reader: ObjectReader);
}
export declare class SpriteSettings {
    readonly settingsRaw: number;
    readonly packed: number;
    readonly packingMode: SpritePackingMode;
    readonly packingRotation: SpritePackingRotation;
    readonly meshType: SpriteMeshType;
    constructor(reader: ObjectReader);
}
export declare class SpriteVertex {
    readonly pos: Vector3;
    readonly uv?: Vector2;
    constructor(reader: ObjectReader);
}
export declare class SpriteRenderData {
    readonly texture: PPtr<Texture2D>;
    readonly alphaTexture?: PPtr<Texture2D>;
    readonly seconaryTextures?: SecondarySpriteTexture[];
    readonly subMeshes?: SubMesh[];
    readonly indexBuffer?: number[];
    readonly vertexData?: VertexData;
    readonly vertices?: SpriteVertex[];
    readonly indices?: number[];
    readonly bindPose?: Matrix4x4[];
    readonly sourceSkin?: BoneWeights4[];
    readonly textureRect: Rectf;
    readonly textureRectOffset: Vector2;
    readonly atlasRectOffset?: Vector2;
    readonly settingsRaw: SpriteSettings;
    readonly uvTransform?: Vector4;
    readonly downscaleMultiplier?: number;
    constructor(reader: ObjectReader);
}
export declare class SpriteAtlas {
    constructor(reader: ObjectReader);
}
export declare class Sprite extends NamedObject {
    readonly rect: Rectf;
    readonly offset: Vector2;
    readonly border?: Vector4;
    readonly pixelsToUnits: number;
    readonly pivot?: Vector2;
    readonly extrude: number;
    readonly isPolygon?: boolean;
    readonly renderDataKey?: (bigint | Buffer)[];
    readonly atlasTags?: string[];
    readonly spriteAtlas?: PPtr<SpriteAtlas>;
    readonly spriteRenderData: SpriteRenderData;
    readonly physicsShape?: Vector2[][];
    constructor(reader: ObjectReader);
}
export declare function isSprite(obj: BaseObject): obj is Sprite;
