"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isSprite = exports.Sprite = exports.SpriteAtlas = exports.SpriteRenderData = exports.SpriteVertex = exports.SpriteSettings = exports.SecondarySpriteTexture = exports.Rectf = exports.SpriteMeshType = exports.SpritePackingMode = exports.SpritePackingRotation = void 0;
const BuildType_1 = require("../BuildType");
const PPtr_1 = require("./PPtr");
const VersionTuple_1 = require("../VersionTuple");
const Matrix_1 = require("./Matrix");
const Mesh_1 = require("./Mesh");
const NamedObject_1 = require("./NamedObject");
const Texture2D_1 = require("./Texture2D");
const Vector_1 = require("./Vector");
const ClassIdType_1 = require("../ClassIdType");
var SpritePackingRotation;
(function (SpritePackingRotation) {
    SpritePackingRotation[SpritePackingRotation["kSPRNone"] = 0] = "kSPRNone";
    SpritePackingRotation[SpritePackingRotation["kSPRFlipHorizontal"] = 1] = "kSPRFlipHorizontal";
    SpritePackingRotation[SpritePackingRotation["kSPRFlipVertical"] = 2] = "kSPRFlipVertical";
    SpritePackingRotation[SpritePackingRotation["kSPRRotate180"] = 3] = "kSPRRotate180";
    SpritePackingRotation[SpritePackingRotation["kSPRRotate90"] = 4] = "kSPRRotate90";
})(SpritePackingRotation = exports.SpritePackingRotation || (exports.SpritePackingRotation = {}));
var SpritePackingMode;
(function (SpritePackingMode) {
    SpritePackingMode[SpritePackingMode["kSPMTight"] = 0] = "kSPMTight";
    SpritePackingMode[SpritePackingMode["kSPMRectangle"] = 1] = "kSPMRectangle";
})(SpritePackingMode = exports.SpritePackingMode || (exports.SpritePackingMode = {}));
var SpriteMeshType;
(function (SpriteMeshType) {
    SpriteMeshType[SpriteMeshType["kSpriteMeshTypeFullRect"] = 0] = "kSpriteMeshTypeFullRect";
    SpriteMeshType[SpriteMeshType["kSpriteMeshTypeTight"] = 1] = "kSpriteMeshTypeTight";
})(SpriteMeshType = exports.SpriteMeshType || (exports.SpriteMeshType = {}));
class Rectf {
    constructor(reader) {
        this.x = reader.readSingle();
        this.y = reader.readSingle();
        this.width = reader.readSingle();
        this.height = reader.readSingle();
    }
}
exports.Rectf = Rectf;
class SecondarySpriteTexture {
    constructor(reader) {
        this.texture = new PPtr_1.PPtr(reader, Texture2D_1.Texture2D);
        this.name = reader.readStringNT();
    }
}
exports.SecondarySpriteTexture = SecondarySpriteTexture;
class SpriteSettings {
    constructor(reader) {
        this.settingsRaw = reader.readUInt32();
        this.packed = this.settingsRaw & 1; //1
        this.packingMode = (this.settingsRaw >> 1) & 1; //1
        this.packingRotation = (this.settingsRaw >> 2) & 0xf; //4
        this.meshType = (this.settingsRaw >> 6) & 1; //1
        //reserved
    }
}
exports.SpriteSettings = SpriteSettings;
class SpriteVertex {
    constructor(reader) {
        var version = reader.version;
        this.pos = Vector_1.readVector3(reader);
        if (version <= VersionTuple_1.makeVersion(4, 3)) {
            //4.3 and down
            this.uv = Vector_1.readVector2(reader);
        }
    }
}
exports.SpriteVertex = SpriteVertex;
class SpriteRenderData {
    constructor(reader) {
        const version = reader.version;
        this.texture = new PPtr_1.PPtr(reader, Texture2D_1.Texture2D);
        if (version >= VersionTuple_1.makeVersion(5, 2)) {
            this.alphaTexture = new PPtr_1.PPtr(reader, Texture2D_1.Texture2D);
        }
        if (version >= VersionTuple_1.makeVersion(2019)) {
            this.seconaryTextures = reader.readObjectArray(SecondarySpriteTexture);
        }
        if (version >= VersionTuple_1.makeVersion(5, 6)) {
            this.subMeshes = reader.readObjectArray(Mesh_1.SubMesh);
            this.indexBuffer = reader.readArray('readUInt8');
            reader.alignStream();
            this.vertexData = new Mesh_1.VertexData(reader);
        }
        else {
            this.vertices = reader.readObjectArray(SpriteVertex);
            this.indices = reader.readArray('readUInt16');
            reader.alignStream();
        }
        if (version > VersionTuple_1.makeVersion(2018)) {
            this.bindPose = reader.readDelegatedArray(Matrix_1.readMatrix4x4);
            if (version <= VersionTuple_1.makeVersion(2018, 2)) {
                this.sourceSkin = reader.readObjectArray(Mesh_1.BoneWeights4);
            }
        }
        this.textureRect = new Rectf(reader);
        this.textureRectOffset = Vector_1.readVector2(reader);
        if (version >= VersionTuple_1.makeVersion(5, 6)) {
            this.atlasRectOffset = Vector_1.readVector2(reader);
        }
        this.settingsRaw = new SpriteSettings(reader);
        if (version >= VersionTuple_1.makeVersion(4, 5)) {
            this.uvTransform = Vector_1.readVector4(reader);
        }
        if (version >= VersionTuple_1.makeVersion(2017)) {
            this.downscaleMultiplier = reader.readSingle();
        }
    }
}
exports.SpriteRenderData = SpriteRenderData;
class SpriteAtlas {
    constructor(reader) { }
}
exports.SpriteAtlas = SpriteAtlas;
class Sprite extends NamedObject_1.NamedObject {
    constructor(reader) {
        super(reader);
        this.rect = new Rectf(reader);
        this.offset = Vector_1.readVector2(reader);
        if (this.version >= VersionTuple_1.makeVersion(4, 5)) {
            this.border = Vector_1.readVector4(reader);
        }
        this.pixelsToUnits = reader.readSingle();
        if (this.version >= VersionTuple_1.makeVersion(5, 4, 2) ||
            (this.version >= VersionTuple_1.makeVersion(5, 4, 1, 3) && this.buildType === BuildType_1.BuildType.Patch)) {
            this.pivot = Vector_1.readVector2(reader);
        }
        this.extrude = reader.readUInt32();
        if (this.version >= VersionTuple_1.makeVersion(5, 3)) {
            this.isPolygon = reader.readBoolean();
            reader.alignStream();
        }
        if (this.version >= VersionTuple_1.makeVersion(2017)) {
            this.renderDataKey = [reader.readBuffer(16), reader.readInt64()];
            this.atlasTags = reader.readArray('readAlignedString');
            this.spriteAtlas = new PPtr_1.PPtr(reader, SpriteAtlas);
        }
        this.spriteRenderData = new SpriteRenderData(reader);
        if (this.version >= VersionTuple_1.makeVersion(2017)) {
            this.physicsShape = reader.readDelegatedArray((reader) => reader.readDelegatedArray(Vector_1.readVector2));
        }
    }
}
exports.Sprite = Sprite;
function isSprite(obj) {
    return obj.classId === ClassIdType_1.ClassIDType.Sprite;
}
exports.isSprite = isSprite;
//# sourceMappingURL=Sprite.js.map