"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTexture2D = exports.isTexture = exports.Texture2D = exports.StreamingInfo = exports.GLTextureSettings = exports.Texture = exports.TextureFormat = void 0;
const ClassIdType_1 = require("../ClassIdType");
const ResourceReader_1 = require("../ResourceReader");
const VersionTuple_1 = require("../VersionTuple");
const NamedObject_1 = require("./NamedObject");
var TextureFormat;
(function (TextureFormat) {
    TextureFormat[TextureFormat["Alpha8"] = 1] = "Alpha8";
    TextureFormat[TextureFormat["ARGB4444"] = 2] = "ARGB4444";
    TextureFormat[TextureFormat["RGB24"] = 3] = "RGB24";
    TextureFormat[TextureFormat["RGBA32"] = 4] = "RGBA32";
    TextureFormat[TextureFormat["ARGB32"] = 5] = "ARGB32";
    TextureFormat[TextureFormat["RGB565"] = 7] = "RGB565";
    TextureFormat[TextureFormat["R16"] = 9] = "R16";
    TextureFormat[TextureFormat["DXT1"] = 10] = "DXT1";
    TextureFormat[TextureFormat["DXT5"] = 12] = "DXT5";
    TextureFormat[TextureFormat["RGBA4444"] = 13] = "RGBA4444";
    TextureFormat[TextureFormat["BGRA32"] = 14] = "BGRA32";
    TextureFormat[TextureFormat["RHalf"] = 15] = "RHalf";
    TextureFormat[TextureFormat["RGHalf"] = 16] = "RGHalf";
    TextureFormat[TextureFormat["RGBAHalf"] = 17] = "RGBAHalf";
    TextureFormat[TextureFormat["RFloat"] = 18] = "RFloat";
    TextureFormat[TextureFormat["RGFloat"] = 19] = "RGFloat";
    TextureFormat[TextureFormat["RGBAFloat"] = 20] = "RGBAFloat";
    TextureFormat[TextureFormat["YUY2"] = 21] = "YUY2";
    TextureFormat[TextureFormat["RGB9e5Float"] = 22] = "RGB9e5Float";
    TextureFormat[TextureFormat["BC4"] = 26] = "BC4";
    TextureFormat[TextureFormat["BC5"] = 27] = "BC5";
    TextureFormat[TextureFormat["BC6H"] = 24] = "BC6H";
    TextureFormat[TextureFormat["BC7"] = 25] = "BC7";
    TextureFormat[TextureFormat["DXT1Crunched"] = 28] = "DXT1Crunched";
    TextureFormat[TextureFormat["DXT5Crunched"] = 29] = "DXT5Crunched";
    TextureFormat[TextureFormat["PVRTC_RGB2"] = 30] = "PVRTC_RGB2";
    TextureFormat[TextureFormat["PVRTC_RGBA2"] = 31] = "PVRTC_RGBA2";
    TextureFormat[TextureFormat["PVRTC_RGB4"] = 32] = "PVRTC_RGB4";
    TextureFormat[TextureFormat["PVRTC_RGBA4"] = 33] = "PVRTC_RGBA4";
    TextureFormat[TextureFormat["ETC_RGB4"] = 34] = "ETC_RGB4";
    TextureFormat[TextureFormat["ATC_RGB4"] = 35] = "ATC_RGB4";
    TextureFormat[TextureFormat["ATC_RGBA8"] = 36] = "ATC_RGBA8";
    TextureFormat[TextureFormat["EAC_R"] = 41] = "EAC_R";
    TextureFormat[TextureFormat["EAC_R_SIGNED"] = 42] = "EAC_R_SIGNED";
    TextureFormat[TextureFormat["EAC_RG"] = 43] = "EAC_RG";
    TextureFormat[TextureFormat["EAC_RG_SIGNED"] = 44] = "EAC_RG_SIGNED";
    TextureFormat[TextureFormat["ETC2_RGB"] = 45] = "ETC2_RGB";
    TextureFormat[TextureFormat["ETC2_RGBA1"] = 46] = "ETC2_RGBA1";
    TextureFormat[TextureFormat["ETC2_RGBA8"] = 47] = "ETC2_RGBA8";
    TextureFormat[TextureFormat["ASTC_RGB_4x4"] = 48] = "ASTC_RGB_4x4";
    TextureFormat[TextureFormat["ASTC_RGB_5x5"] = 49] = "ASTC_RGB_5x5";
    TextureFormat[TextureFormat["ASTC_RGB_6x6"] = 50] = "ASTC_RGB_6x6";
    TextureFormat[TextureFormat["ASTC_RGB_8x8"] = 51] = "ASTC_RGB_8x8";
    TextureFormat[TextureFormat["ASTC_RGB_10x10"] = 52] = "ASTC_RGB_10x10";
    TextureFormat[TextureFormat["ASTC_RGB_12x12"] = 53] = "ASTC_RGB_12x12";
    TextureFormat[TextureFormat["ASTC_RGBA_4x4"] = 54] = "ASTC_RGBA_4x4";
    TextureFormat[TextureFormat["ASTC_RGBA_5x5"] = 55] = "ASTC_RGBA_5x5";
    TextureFormat[TextureFormat["ASTC_RGBA_6x6"] = 56] = "ASTC_RGBA_6x6";
    TextureFormat[TextureFormat["ASTC_RGBA_8x8"] = 57] = "ASTC_RGBA_8x8";
    TextureFormat[TextureFormat["ASTC_RGBA_10x10"] = 58] = "ASTC_RGBA_10x10";
    TextureFormat[TextureFormat["ASTC_RGBA_12x12"] = 59] = "ASTC_RGBA_12x12";
    TextureFormat[TextureFormat["ETC_RGB4_3DS"] = 60] = "ETC_RGB4_3DS";
    TextureFormat[TextureFormat["ETC_RGBA8_3DS"] = 61] = "ETC_RGBA8_3DS";
    TextureFormat[TextureFormat["RG16"] = 62] = "RG16";
    TextureFormat[TextureFormat["R8"] = 63] = "R8";
    TextureFormat[TextureFormat["ETC_RGB4Crunched"] = 64] = "ETC_RGB4Crunched";
    TextureFormat[TextureFormat["ETC2_RGBA8Crunched"] = 65] = "ETC2_RGBA8Crunched";
    TextureFormat[TextureFormat["ASTC_HDR_4x4"] = 66] = "ASTC_HDR_4x4";
    TextureFormat[TextureFormat["ASTC_HDR_5x5"] = 67] = "ASTC_HDR_5x5";
    TextureFormat[TextureFormat["ASTC_HDR_6x6"] = 68] = "ASTC_HDR_6x6";
    TextureFormat[TextureFormat["ASTC_HDR_8x8"] = 69] = "ASTC_HDR_8x8";
    TextureFormat[TextureFormat["ASTC_HDR_10x10"] = 70] = "ASTC_HDR_10x10";
    TextureFormat[TextureFormat["ASTC_HDR_12x12"] = 71] = "ASTC_HDR_12x12";
})(TextureFormat = exports.TextureFormat || (exports.TextureFormat = {}));
class Texture extends NamedObject_1.NamedObject {
    constructor(reader) {
        super(reader);
        if (this.version >= VersionTuple_1.makeVersion(2017, 3)) {
            //2017.3+
            reader.readInt32(); // forcedFallbackFormat
            reader.readBoolean(); // downscaleFallback
            if (this.version >= VersionTuple_1.makeVersion(2020, 2)) {
                //2020.2+
                reader.readBoolean(); // isAlphaChannelOptional
            }
            reader.alignStream();
        }
    }
}
exports.Texture = Texture;
class GLTextureSettings {
    constructor(reader) {
        const version = reader.version;
        this.filterMode = reader.readInt32();
        this.aniso = reader.readInt32();
        this.mipBias = reader.readSingle();
        if (version >= VersionTuple_1.makeVersion(2017)) {
            //2017.x and up
            this.wrapMode = reader.readInt32(); // m_WrapU
            reader.readInt32(); // m_WrapV
            reader.readInt32(); // m_WrapW
        }
        else {
            this.wrapMode = reader.readInt32();
        }
    }
}
exports.GLTextureSettings = GLTextureSettings;
class StreamingInfo {
    constructor(reader) {
        const version = reader.version;
        if (version >= VersionTuple_1.makeVersion(2020)) {
            // 2020.1+
            this.offset = reader.readUInt64();
        }
        else {
            this.offset = reader.readUInt32();
        }
        this.size = reader.readUInt32();
        this.path = reader.readAlignedString();
    }
}
exports.StreamingInfo = StreamingInfo;
class Texture2D extends Texture {
    constructor(reader) {
        var _a;
        super(reader);
        this.width = reader.readInt32();
        this.height = reader.readInt32();
        reader.readInt32(); // completeImageSize
        if (this.version >= VersionTuple_1.makeVersion(2020)) {
            // 2020.1+
            reader.readInt32(); //mipsStripped
        }
        this.textureFormat = reader.readInt32();
        if (this.version < VersionTuple_1.makeVersion(5, 2)) {
            this.mipMap = reader.readBoolean();
        }
        else {
            this.mipCount = reader.readInt32();
        }
        if (this.version >= VersionTuple_1.makeVersion(2, 6)) {
            //2.6.0 and up
            reader.readBoolean(); // m_IsReadable
        }
        if (this.version >= VersionTuple_1.makeVersion(2020)) {
            //2020.1 and up
            reader.readBoolean(); // m_IsPreProcessed
        }
        if (this.version >= VersionTuple_1.makeVersion(2019, 3)) {
            //2019.3 and up
            reader.readBoolean(); // m_IgnoreMasterTextureLimit
        }
        if (this.version >= VersionTuple_1.makeVersion(3) && this.version <= VersionTuple_1.makeVersion(5, 4)) {
            //3.0.0 - 5.4
            reader.readBoolean(); // m_ReadAllowed
        }
        if (this.version >= VersionTuple_1.makeVersion(2018, 2)) {
            //2018.2 and up
            reader.readBoolean(); // m_StreamingMipmaps
        }
        reader.alignStream();
        if (this.version >= VersionTuple_1.makeVersion(2018, 2)) {
            //2018.2 and up
            reader.readInt32(); // m_StreamingMipmapsPriority
        }
        reader.readInt32(); // m_ImageCount
        reader.readInt32(); // m_TextureDimension
        this.textureSettings = new GLTextureSettings(reader);
        if (this.version >= VersionTuple_1.makeVersion(3)) {
            //3.0 and up
            reader.readInt32(); // m_LightmapFormat
        }
        if (this.version >= VersionTuple_1.makeVersion(3, 5)) {
            //3.5.0 and up
            reader.readInt32(); // m_ColorSpace
        }
        if (this.version >= VersionTuple_1.makeVersion(2020, 2)) {
            //2020.2 and up
            reader.readArray('readUInt8'); // m_PlatformBlob
            reader.alignStream();
        }
        const imageDataSize = reader.readInt32();
        if (imageDataSize == 0 && this.version > VersionTuple_1.makeVersion(5, 3)) {
            //5.3.0 and up
            this.streamData = new StreamingInfo(reader);
        }
        if (this.streamData && this.streamData.path) {
            this.resourceReader = new ResourceReader_1.ResourceReader((_a = this.streamData) === null || _a === void 0 ? void 0 : _a.path, reader.assetsFile, this.streamData.offset, this.streamData.size);
        }
        else {
            this.resourceReader = new ResourceReader_1.ResourceReader(reader, reader.readOffset, imageDataSize);
        }
    }
}
exports.Texture2D = Texture2D;
function isTexture(obj) {
    return obj.classId === ClassIdType_1.ClassIDType.Texture || obj.classId === ClassIdType_1.ClassIDType.Texture2D;
}
exports.isTexture = isTexture;
function isTexture2D(obj) {
    return obj.classId === ClassIdType_1.ClassIDType.Texture2D;
}
exports.isTexture2D = isTexture2D;
//# sourceMappingURL=Texture2D.js.map