"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResourceReader = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const Assert_1 = require("../utils/Assert");
class ResourceReader {
    constructor(...args) {
        this.needSearch = false;
        if (args.length === 4) {
            const [path, assetFile, offset, size] = args;
            this.path = path;
            this.assetFile = assetFile;
            this.offset = offset;
            this.size = size;
            this.needSearch = true;
        }
        else {
            const [reader, offset, size] = args;
            this.reader = reader;
            this.offset = offset;
            this.size = size;
            this.needSearch = false;
        }
    }
    getData() {
        Assert_1.assert(this.assetFile && this.path);
        if (this.needSearch) {
            const resourceFileName = path_1.default.basename(this.path);
            this.reader = this.assetFile.assetManager.resourceFileReaders.get(resourceFileName);
            if (!this.reader) {
                throw new Error(`Can't find the resource file ${resourceFileName}`);
            }
            this.needSearch = false;
        }
        Assert_1.assert(this.reader);
        this.reader.readOffset = Number(this.offset);
        return this.reader.readBuffer(this.size);
    }
}
exports.ResourceReader = ResourceReader;
//# sourceMappingURL=ResourceReader.js.map