/// <reference types="node" />
/// <reference types="ref-napi" />
import type { AssetManager } from './AssetManager';
import { BuildTarget } from './BuildTarget';
import { BuildType } from './BuildType';
import { ClassIDType } from './ClassIdType';
import { EndianReader } from './EndianReader';
import { BaseObject } from './Objects/BaseObject';
import { Version } from './VersionTuple';
interface SerializedFileHeader {
    metadataSize: number;
    fileSize: number | bigint;
    version: number;
    dataOffset: number | bigint;
    endianness: number;
}
export interface TypeTreeNode {
    type: string;
    name: string;
    byteSize: number;
    index?: number;
    isArray: boolean;
    version: number;
    metaFlag?: number;
    level: number;
}
export interface ObjectInfo {
    byteStart: number | bigint;
    byteSize: number;
    typeId: number;
    classId: ClassIDType;
    pathId: number | bigint;
    serializedType?: SerializedType;
}
export interface TypeTreeNodeBlob extends TypeTreeNode {
    typeStrOffset: number;
    nameStrOffset: number;
}
export interface SerializedType {
    classId: ClassIDType;
    isStrippedType?: boolean;
    scriptTypeIndex?: number;
    scriptId?: Buffer;
    oldTypeHash?: Buffer;
    nodes?: TypeTreeNode[];
    typeDependencies?: number[];
}
export declare function isSerializedFile(reader: EndianReader): boolean;
export declare class SerializedFile {
    readonly assetManager: AssetManager;
    readonly name: string;
    readonly bundlePrefix?: string | undefined;
    private enableTypeTree;
    private bigIDEnabled;
    readonly header: SerializedFileHeader;
    readonly reader: EndianReader;
    version: Version;
    unityVersion: string;
    targetPlatform: BuildTarget;
    readonly types: SerializedType[];
    readonly objectInfos: ObjectInfo[];
    readonly objects: Map<number | bigint, BaseObject>;
    buildType: BuildType;
    constructor(assetManager: AssetManager, name: string, reader: EndianReader, bundlePrefix?: string | undefined);
    setVersion(version: string): void;
    private readHeader;
    private readMetadata;
    private readTypes;
    private readSerializedType;
    private typeTreeBlobRead;
    private readTypeTree;
    private readObjects;
}
export {};
