"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.invokeDecoder = exports.getTextureBuffer = void 0;
const tslib_1 = require("tslib");
const ffi_napi_1 = require("ffi-napi");
const Texture2D_1 = require("./Objects/Texture2D");
const path_1 = tslib_1.__importDefault(require("path"));
const Logger_1 = require("./Logger");
process.env.PATH = [process.env.PATH, path_1.default.resolve(__dirname, '../../bin')].join(';');
const TextureDecoder = ffi_napi_1.Library('Texture2DDecoderNative', {
    DecodeDXT1: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeDXT5: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeBC4: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeBC5: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeBC6: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeBC7: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeETC1: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeATCRGB4: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeATCRGBA8: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeEACR: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeEACRSigned: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeEACRG: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeETC2: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeETC2A1: ['bool', ['byte*', 'int', 'int', 'byte*']],
    DecodeETC2A8: ['bool', ['byte*', 'int', 'int', 'byte*']],
});
async function getTextureBuffer(texture) {
    const formatLogStr = `${texture.textureFormat}(${Texture2D_1.TextureFormat[texture.textureFormat]})`;
    Logger_1.log(`Loading texture '${texture.name}' type=${formatLogStr}`);
    switch (texture.textureFormat) {
        case Texture2D_1.TextureFormat.DXT1:
            return invokeDecoder(texture, 'DecodeDXT1');
        case Texture2D_1.TextureFormat.DXT5:
            return invokeDecoder(texture, 'DecodeDXT5');
        case Texture2D_1.TextureFormat.BC4:
            return invokeDecoder(texture, 'DecodeBC4');
        case Texture2D_1.TextureFormat.BC5:
            return invokeDecoder(texture, 'DecodeBC5');
        case Texture2D_1.TextureFormat.BC6H:
            return invokeDecoder(texture, 'DecodeBC6');
        case Texture2D_1.TextureFormat.BC7:
            return invokeDecoder(texture, 'DecodeBC7');
        case Texture2D_1.TextureFormat.ETC_RGB4:
        case Texture2D_1.TextureFormat.ETC_RGB4_3DS:
            return invokeDecoder(texture, 'DecodeETC1');
        case Texture2D_1.TextureFormat.ATC_RGB4:
            return invokeDecoder(texture, 'DecodeATCRGB4');
        case Texture2D_1.TextureFormat.ATC_RGBA8:
            return invokeDecoder(texture, 'DecodeATCRGBA8');
        case Texture2D_1.TextureFormat.EAC_R:
            return invokeDecoder(texture, 'DecodeEACR');
        case Texture2D_1.TextureFormat.EAC_R_SIGNED:
            return invokeDecoder(texture, 'DecodeEACRSigned');
        case Texture2D_1.TextureFormat.EAC_RG:
            return invokeDecoder(texture, 'DecodeEACRG');
        case Texture2D_1.TextureFormat.ETC2_RGB:
            return invokeDecoder(texture, 'DecodeETC2');
        case Texture2D_1.TextureFormat.ETC2_RGBA1:
            return invokeDecoder(texture, 'DecodeETC2A1');
        case Texture2D_1.TextureFormat.ETC2_RGBA8:
        case Texture2D_1.TextureFormat.ETC_RGBA8_3DS:
            return invokeDecoder(texture, 'DecodeETC2A8');
        default:
            throw new Error(`Unsupported format ${formatLogStr}`);
    }
}
exports.getTextureBuffer = getTextureBuffer;
async function invokeDecoder(texture, format) {
    const outBuffer = Buffer.alloc(texture.width * texture.height * 4);
    const sourceBuffer = texture.resourceReader.getData();
    const result = await new Promise((resolve, reject) => {
        try {
            TextureDecoder[format].async(sourceBuffer, texture.width, texture.height, outBuffer, (err, result) => (err ? reject(err) : resolve(result)));
        }
        catch (e) {
            reject(e);
        }
    });
    if (!result) {
        throw new Error(`An error ocurred decoding texture '${texture.name}' with ${format}`);
    }
    return outBuffer;
}
exports.invokeDecoder = invokeDecoder;
//# sourceMappingURL=TextureDecoder.js.map