"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLogger = void 0;
const tslib_1 = require("tslib");
const tracer_1 = require("tracer");
const StripAnsi_1 = require("./StripAnsi");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
let logFile;
let isInitialized = false;
function initLogFile() {
    if (isInitialized) {
        return logFile;
    }
    isInitialized = true;
    const logDir = path_1.default.join(process.cwd(), 'logs');
    console.info(`Initializing log directory: ${logDir}`);
    try {
        if (!fs_1.default.existsSync(logDir)) {
            fs_1.default.mkdirSync(logDir, { recursive: true });
        }
        logFile = path_1.default.join(logDir, `log_${process.pid}_${new Date().toISOString().replace(/[:\-]/g, '.')}.log`);
    }
    catch (_a) { }
    return logFile;
}
function createLogger() {
    return tracer_1.colorConsole({
        transport(data) {
            const logFile = initLogFile();
            if (logFile) {
                fs_1.default.appendFile(logFile, StripAnsi_1.stripAnsi(data.rawoutput) + '\n', () => { });
            }
        },
    });
}
exports.createLogger = createLogger;
//# sourceMappingURL=CreateLogger.js.map