"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.debug = exports.setLogFn = void 0;
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const performance_1 = require("@perf-tools/performance");
const pretty_ms_1 = tslib_1.__importDefault(require("pretty-ms"));
const CreateLogger_1 = require("./CreateLogger");
const logger = CreateLogger_1.createLogger();
const env = process.env;
const debugSettings = new Set(env['DEBUG'] ? env['DEBUG'].split(',') : []);
function hashString(value) {
    let hash = 0;
    if (!value.length) {
        return hash;
    }
    for (let i = 0; i < value.length; ++i) {
        hash = ((hash << 5) - hash + value.charCodeAt(i)) | 0;
    }
    return hash;
}
const frequency = 0.5;
const width = 100;
const offset = 155;
function rainbow(index) {
    return chalk_1.default.rgb(Math.floor(Math.sin(frequency * index + 0) * width + offset), Math.floor(Math.sin(frequency * index + 2) * width + offset), Math.floor(Math.sin(frequency * index + 4) * width + offset));
}
function hashColor(name) {
    return rainbow(hashString(name));
}
let logFn = console.log.bind(console);
if (logger) {
    logFn = logger.log.bind(logger);
}
function setLogFn(fn) {
    logFn = fn;
}
exports.setLogFn = setLogFn;
let last = undefined;
function debug(name, message, ...args) {
    if (message === undefined) {
        return debug.bind(undefined, name);
    }
    let delta = '';
    const current = performance_1.now();
    if (last) {
        delta = chalk_1.default.bold(chalk_1.default.cyanBright(pretty_ms_1.default(current - last)));
    }
    last = current;
    if (debugSettings.has(name) || logger) {
        logFn(`${hashColor(name)(`[${name}]`)} ${message}`, ...args, delta);
    }
}
exports.debug = debug;
//# sourceMappingURL=DebugLog.js.map