"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cache = exports.setCacheDir = void 0;
const tslib_1 = require("tslib");
const cacache_1 = tslib_1.__importDefault(require("cacache"));
const is_promise_1 = tslib_1.__importDefault(require("is-promise"));
class CacheStatics {
    setCacheDir(cacheDir) {
        this.cacheDir = cacheDir;
    }
}
const globalCache = new CacheStatics();
function setCacheDir(dir) {
    globalCache.setCacheDir(dir);
}
exports.setCacheDir = setCacheDir;
class Cache {
    constructor(name, fetch, serializer) {
        this.name = name;
        this.fetch = fetch;
        this.serializer = serializer;
        this.memCache = new Map();
    }
    async get(key, cachedOnly) {
        const contentKey = `${this.name}\0${key}`;
        try {
            if (globalCache.cacheDir) {
                const result = (await cacache_1.default.get(globalCache.cacheDir, contentKey)).data;
                if (this.serializer) {
                    const parsed = this.serializer.parse(result);
                    return parsed;
                }
                return result;
            }
        }
        catch (_a) { }
        const cachedResult = this.memCache.get(contentKey);
        if (cachedResult) {
            return cachedResult;
        }
        if (cachedOnly) {
            return undefined;
        }
        let result = this.fetch(key);
        if (is_promise_1.default(result)) {
            result = await result;
        }
        try {
            if (globalCache.cacheDir) {
                if (this.serializer) {
                    const stream = this.serializer.stringify(result);
                    cacache_1.default.put(globalCache.cacheDir, contentKey, stream, { memoize: true });
                }
                else {
                    cacache_1.default.put(globalCache.cacheDir, contentKey, result, { memoize: true });
                }
                return result;
            }
        }
        catch (_b) { }
        this.memCache.set(key, result);
        return result;
    }
}
exports.Cache = Cache;
//# sourceMappingURL=ReadThruCache.js.map